/*
 *	id: @(#)fakeboot.c 2.3 93/03/10
 *	purpose: 
 *	copyright: Copyright 1990 Sun Microsystems, Inc.  All Rights Reserved
 */
/*
 * Program to enclose a text file in an a.out or elf32 format file, 
 * so that the file can be loaded into memory with the boot program.
 * The text begins at location 0x4010, and its size is stored at
 * location 0x4004.  Location 0x4000, which is the "entry point"
 * of the loadable file, contains an illegal instruction, which will
 * cause a trap back to the monitor.
 *
 * Compilation: make fakeboot
 *
 * Usage:  fakeboot [-a.out|-elf32 ] <textfile >binaryfile
 * the default format is a.out.
 *
 * Monitor command for loading the file, assuming that "binaryfile"
 * in the root directory:
 *
 *     boot binaryfile -h
 *
 * Then, when the file is loaded, to interpret it as Forth source code, type:
 *
 *     4010 4004 @ eval
 */

#include <stdio.h>
#include <sys/file.h>
#include "elf.h"

/*
* For some unknown reason, the SunOS 4.X boot program requires program 
* files to be at least 0x2000 long.
*/
#define AOUT_HDR_SIZE  0x20
#define EVAL_INFO_SIZE 0x10
#define MINSIZE	       0x2000 - AOUT_HDR_SIZE - EVAL_INFO_SIZE

#define ENTRY_PT       0x4000

main(argc, argv)
	int argc;
	char **argv;
{
	int data_size, big_size, residule;
	int format;

        if (argc > 2 ) usage();

	if (argc == 1 ) { 
	  format = 0;
	  fprintf(stderr, "Defaulting to a.out format \n");
	}
	else if (strcmp(argv[1], "-elf32") == 0) format = 1;
	else if (strcmp(argv[1], "-a.out") == 0) format = 0;
	else usage();

	/* Initialize stdin */
	data_size = lseek(0, 0, L_XTND);
	(void) lseek(0, 0, L_SET);
	
	switch (format) {
	case 0:
	    /* Calculate output data size based on 4.X restriction */
   	    big_size = (data_size >= MINSIZE) ? data_size : MINSIZE;
	    residule = ((big_size + 7) & ~7) - data_size;
	    put_aout_hdr(big_size+EVAL_INFO_SIZE);
	    put_eval_info(data_size);
	    while (data_size--)
		putchar(getchar());
	    while (residule--)
		putchar(0);
	    break;

	case 1:
	    put_elf_hdr(data_size+EVAL_INFO_SIZE);
	    put_eval_info(data_size);
	    while (data_size--)
		putchar(getchar());
	    break;
	}
	exit(0);
}

put_eval_info(size)
     int size;
{
	/* File size info for eval */
	putlong (0);	/* Illegal instruction */
	putlong (size);
	putlong (0);	/* Reserved */
	putlong (0);	/* Reserved */
}

put_aout_hdr(size)
     int size;
{        
	putlong (0x01030107); putlong (size); putlong (0);  putlong (0);
	putlong (0);	      putlong (0x4000);     putlong (0);  putlong (0);

}

put_elf_hdr(size)
     int size;
{
	/* ELF file header */
	putchar(ELFMAG0); putchar(ELFMAG1); putchar(ELFMAG2); putchar(ELFMAG3); 
	putchar(ELFCLASS32); putchar(ELFDATA2MSB); putchar(EV_CURRENT);
	putchar(0); putchar(0); putchar(0); putchar(0); putchar(0);
	putchar(0); putchar(0); putchar(0); putchar(0);

	putshort(ET_EXEC); putshort(EM_SPARC); putlong(EV_CURRENT);
	putlong(ENTRY_PT);
	putlong(sizeof(Elf32_Ehdr)); putlong(0);               /* e_phoff e_shoff */
	putlong(0); putshort(sizeof(Elf32_Ehdr));              /* e_flags e_ehsize */
	putshort(sizeof(Elf32_Phdr)); putshort(1);             /* e_phentsize e_phnum */
	putshort(sizeof(Elf32_Shdr)); putshort(0);             /* e_shentsize e_shnum */           putshort(0);                                           /* e_shstrndx */

        /* Program header */
        putlong(PT_LOAD); putlong(sizeof(Elf32_Ehdr)+sizeof(Elf32_Phdr));
        putlong(ENTRY_PT); putlong(0);
        putlong(size); putlong(size);
        putlong(PF_R|PF_X); putlong(4);
}

usage()
{
    fprintf(stderr, "Usage:  fakeboot [-a.out|-elf32] <infile >outfile\n");
    exit(1);
}
putlong(l)
	long l;
{
	putchar(l>>24);  putchar(l>>16);  putchar(l>>8);  putchar(l);
}
putshort(s)
	short s;
{
	putchar(s>>8);  putchar(s);
}




